@@ -5,7 +5,7 @@
 
 /*****************************************************************************/
 /*  cdavl.c - CUI disk allocation viewer                                     */
-/*  Copyright: Copyright (c) Hitachi, Ltd. 2004-2007                         */
+/*  Copyright: Copyright (c) Hitachi, Ltd. 2004-2008                         */
 /*             Authors: Yumiko Sugita (yumiko.sugita.yf@hitachi.com),        */
 /*                      Satoshi Fujiwara (sa-fuji@sdl.hitachi.co.jp)         */
 /*                                                                           */
@@ -39,6 +39,15 @@
 #include "../common/util.h"
 #include "../drv/liveinfo_app.h"
 #include "cdavl.h"
+#include <endian.h>
+#if BYTE_ORDER == LITTLE_ENDIAN
+#  include <linux/byteorder/little_endian.h>
+#else
+#  include <linux/byteorder/big_endian.h>
+#endif
+
+#define le16_to_cpu	__le16_to_cpu
+#define le32_to_cpu	__le32_to_cpu
 
 
 
@@ -82,57 +91,57 @@ void printSuperBlock(t_e2info* fs) {
 	struct ext2_super_block	*sb = &fs->sb;
 
 	printf("------ super block ------\n");
-	PRINT4("s_inodes_count:           ", sb->s_inodes_count);
-	PRINT4("s_blocks_count:           ", sb->s_blocks_count);
-	PRINT4("s_r_blocks_count:         ", sb->s_r_blocks_count);
-	PRINT4("s_free_blocks_count:      ", sb->s_free_blocks_count);
-	PRINT4("s_free_inodes_count:      ", sb->s_free_inodes_count);
-	PRINT4("s_first_data_block:       ", sb->s_first_data_block);
-	PRINT4("s_log_block_size:         ", sb->s_log_block_size);
-	PRINT4("s_log_frag_size:          ", sb->s_log_frag_size);
-	PRINT4("s_blocks_per_group:       ", sb->s_blocks_per_group);
-	PRINT4("s_frags_per_group:        ", sb->s_frags_per_group);
-	PRINT4("s_inodes_per_group:       ", sb->s_inodes_per_group);
-	PRINT4("s_mtime:                  ", sb->s_mtime);
-	PRINT4("s_wtime:                  ", sb->s_wtime);
-	PRINT2("s_mnt_count:              ", sb->s_mnt_count);
-	PRINT2("s_max_mnt_count:          ", sb->s_max_mnt_count);
-	PRINT2("s_magic:                  ", sb->s_magic);
-	PRINT2("s_state:                  ", sb->s_state);
-	PRINT2("s_errors:                 ", sb->s_errors);
-	PRINT2("s_minor_rev_level:        ", sb->s_minor_rev_level);
-	PRINT4("s_lastcheck:              ", sb->s_lastcheck);
-	PRINT4("s_checkinterval:          ", sb->s_checkinterval);
-	PRINT4("s_creator_os:             ", sb->s_creator_os);
-	PRINT4("s_rev_level:              ", sb->s_rev_level);
-	PRINT2("s_def_resuid:             ", sb->s_def_resuid);
-	PRINT2("s_def_resgid:             ", sb->s_def_resgid);
-	PRINT4("s_first_ino:              ", sb->s_first_ino);
-	PRINT2("s_inode_size:             ", sb->s_inode_size);
-	PRINT2("s_block_group_nr:         ", sb->s_block_group_nr);
-	PRINT4("s_feature_compat:         ", sb->s_feature_compat);
-	PRINT4("s_feature_incompat:       ", sb->s_feature_incompat);
-	PRINT4("s_feature_ro_compat:      ", sb->s_feature_ro_compat);
+	PRINT4("s_inodes_count:           ", le32_to_cpu(sb->s_inodes_count));
+	PRINT4("s_blocks_count:           ", le32_to_cpu(sb->s_blocks_count));
+	PRINT4("s_r_blocks_count:         ", le32_to_cpu(sb->s_r_blocks_count));
+	PRINT4("s_free_blocks_count:      ", le32_to_cpu(sb->s_free_blocks_count));
+	PRINT4("s_free_inodes_count:      ", le32_to_cpu(sb->s_free_inodes_count));
+	PRINT4("s_first_data_block:       ", le32_to_cpu(sb->s_first_data_block));
+	PRINT4("s_log_block_size:         ", le32_to_cpu(sb->s_log_block_size));
+	PRINT4("s_log_frag_size:          ", le32_to_cpu(sb->s_log_frag_size));
+	PRINT4("s_blocks_per_group:       ", le32_to_cpu(sb->s_blocks_per_group));
+	PRINT4("s_frags_per_group:        ", le32_to_cpu(sb->s_frags_per_group));
+	PRINT4("s_inodes_per_group:       ", le32_to_cpu(sb->s_inodes_per_group));
+	PRINT4("s_mtime:                  ", le32_to_cpu(sb->s_mtime));
+	PRINT4("s_wtime:                  ", le32_to_cpu(sb->s_wtime));
+	PRINT2("s_mnt_count:              ", le16_to_cpu(sb->s_mnt_count));
+	PRINT2("s_max_mnt_count:          ", le16_to_cpu(sb->s_max_mnt_count));
+	PRINT2("s_magic:                  ", (__u16)le16_to_cpu(sb->s_magic));
+	PRINT2("s_state:                  ", le16_to_cpu(sb->s_state));
+	PRINT2("s_errors:                 ", le16_to_cpu(sb->s_errors));
+	PRINT2("s_minor_rev_level:        ", le16_to_cpu(sb->s_minor_rev_level));
+	PRINT4("s_lastcheck:              ", le32_to_cpu(sb->s_lastcheck));
+	PRINT4("s_checkinterval:          ", le32_to_cpu(sb->s_checkinterval));
+	PRINT4("s_creator_os:             ", le32_to_cpu(sb->s_creator_os));
+	PRINT4("s_rev_level:              ", le32_to_cpu(sb->s_rev_level));
+	PRINT2("s_def_resuid:             ", le16_to_cpu(sb->s_def_resuid));
+	PRINT2("s_def_resgid:             ", le16_to_cpu(sb->s_def_resgid));
+	PRINT4("s_first_ino:              ", le32_to_cpu(sb->s_first_ino));
+	PRINT2("s_inode_size:             ", le16_to_cpu(sb->s_inode_size));
+	PRINT2("s_block_group_nr:         ", le16_to_cpu(sb->s_block_group_nr));
+	PRINT4("s_feature_compat:         ", le32_to_cpu(sb->s_feature_compat));
+	PRINT4("s_feature_incompat:       ", le32_to_cpu(sb->s_feature_incompat));
+	PRINT4("s_feature_ro_compat:      ", le32_to_cpu(sb->s_feature_ro_compat));
 	printf("s_uuid:\n");
 	uDumpN((char*)sb->s_uuid, 16);
 	PRINTS("s_volume_name:            ", sb->s_volume_name, 16);
 	PRINTS("s_last_mounted:           ", sb->s_last_mounted, 64);
-	PRINT4("s_algorithm_usage_bitmap: ", sb->s_algorithm_usage_bitmap);
+	PRINT4("s_algorithm_usage_bitmap: ", le32_to_cpu(sb->s_algorithm_usage_bitmap));
 	PRINT1("s_prealloc_blocks:        ", sb->s_prealloc_blocks);
 	PRINT1("s_prealloc_dir_blocks:    ", sb->s_prealloc_dir_blocks);
-	if (sb->s_feature_compat & EXT3_FEATURE_COMPAT_HAS_JOURNAL) {
+	if (le32_to_cpu(sb->s_feature_compat) & EXT3_FEATURE_COMPAT_HAS_JOURNAL) {
 		printf("s_journal_uuid:\n");
 		uDumpN((char*)sb->s_journal_uuid, 16);
-		PRINT4("s_journal_inum:           ", sb->s_journal_inum);
-		PRINT4("s_journal_dev:            ", sb->s_journal_dev);
-		PRINT4("s_last_orphan:            ", sb->s_last_orphan);
+		PRINT4("s_journal_inum:           ", le32_to_cpu(sb->s_journal_inum));
+		PRINT4("s_journal_dev:            ", le32_to_cpu(sb->s_journal_dev));
+		PRINT4("s_last_orphan:            ", le32_to_cpu(sb->s_last_orphan));
 	}
 #ifdef NEW_KERNEL
 	printf("s_hash_seed:\n");
 	uDumpN((char*)sb->s_hash_seed, 4);
 	PRINT1("s_def_hash_version:       ", sb->s_def_hash_version);
-	PRINT4("s_default_mount_opts:     ", sb->s_default_mount_opts);
-	PRINT4("s_first_meta_bg:          ", sb->s_first_meta_bg);
+	PRINT4("s_default_mount_opts:     ", le32_to_cpu(sb->s_default_mount_opts));
+	PRINT4("s_first_meta_bg:          ", le32_to_cpu(sb->s_first_meta_bg));
 #endif
 	printf("\n");
 }
@@ -146,10 +155,10 @@ int getSuperBlock(t_prog_const* prog_c, t_e2info* fs) {
 		return ERROR;
 
 	/* check s_magic */
-	if (fs->sb.s_magic != EXT2_SUPER_MAGIC) {
+	if ((__u16)le16_to_cpu(fs->sb.s_magic) != EXT2_SUPER_MAGIC) {
 		fprintf(stderr,
 			"It's not ext2/ext3 file system.(s_magic: %08x)\n",
-			fs->sb.s_magic);
+			(__u16)le16_to_cpu(fs->sb.s_magic));
 		return ERROR;
 	}
 
@@ -163,7 +172,7 @@ int getGroupDesc(t_prog_const* prog_c, t_e2info* fs) {
 	int		i, rc, size = sizeof(struct ext2_group_desc);
 	off_t		pos;
 
-	pos = (off_t)(fs->sb.s_first_data_block + 1) * fs->bs;
+	pos = (off_t)(le32_to_cpu(fs->sb.s_first_data_block) + 1) * fs->bs;
 	for (i = 0; i < fs->gc; i++) {
 		rc = uSeekRead(prog_c->fd, pos, &fs->gd[i], size);
 		if (rc < 0)
@@ -261,7 +270,7 @@ unsigned int getSysBlocks(t_e2info* fs) {
 }
 
 unsigned int getInodeTblBlocks(t_e2info* fs) {
-	return fs->sb.s_inodes_per_group * fs->sb.s_inode_size / fs->bs;
+	return le32_to_cpu(fs->sb.s_inodes_per_group) * le16_to_cpu(fs->sb.s_inode_size) / fs->bs;
 }
 
 int chkSysBlocks(t_prog_const* prog_c, t_e2info* fs, t_chk* chk) {
@@ -273,25 +282,25 @@ int chkSysBlocks(t_prog_const* prog_c, t_e2info* fs, t_chk* chk) {
 	for (gn = 0; gn < fs->gc; gn++) {
 		gd = &fs->gd[gn];
 		if (gn != 0)
-			start = fs->sb.s_first_data_block +
-				fs->sb.s_blocks_per_group * gn;
-		end = gd->bg_block_bitmap;
+			start = le32_to_cpu(fs->sb.s_first_data_block) +
+				le32_to_cpu(fs->sb.s_blocks_per_group) * gn;
+		end = le32_to_cpu(gd->bg_block_bitmap);
 
 		for (i = start; i < end; i++)
 			bchk[i].type = BT_FS1;
 		if (setSysBlockInfo(fs, start, end - start) == ERROR)
 			return ERROR;
-		bchk[gd->bg_block_bitmap].type = BT_FS2;
-		if (setSysBlockInfo(fs, gd->bg_block_bitmap, 1) == ERROR)
+		bchk[le32_to_cpu(gd->bg_block_bitmap)].type = BT_FS2;
+		if (setSysBlockInfo(fs, le32_to_cpu(gd->bg_block_bitmap), 1) == ERROR)
 			return ERROR;
-		bchk[gd->bg_inode_bitmap].type = BT_FS2;
-		if (setSysBlockInfo(fs, gd->bg_inode_bitmap, 1) == ERROR)
+		bchk[le32_to_cpu(gd->bg_inode_bitmap)].type = BT_FS2;
+		if (setSysBlockInfo(fs, le32_to_cpu(gd->bg_inode_bitmap), 1) == ERROR)
 			return ERROR;
 
-		j = gd->bg_inode_table;
+		j = le32_to_cpu(gd->bg_inode_table);
 		for (i = 0; i < max; i++)
 			bchk[j++].type = BT_UNUSED_INODE;
-		if (setSysBlockInfo(fs, gd->bg_inode_table, max) == ERROR)
+		if (setSysBlockInfo(fs, le32_to_cpu(gd->bg_inode_table), max) == ERROR)
 			return ERROR;
 	}
 	if (prog_c->verbose_level >= 3)
@@ -373,40 +382,40 @@ void printFileBlockSeq(t_blk_seq* p, int is_fragment) {
 void printInode(unsigned int ino, struct ext2_inode* p) {
 #ifdef	DEBUG
 	printf("------ inode: %d ------\n", ino);
-	PRINT2("i_mode:        ", p->i_mode);
-	PRINT2("i_uid:         ", p->i_uid);
-	PRINT4("i_size:        ", p->i_size);
-	PRINT4("i_atime:       ", p->i_atime);
-	PRINT4("i_ctime:       ", p->i_ctime);
-	PRINT4("i_mtime:       ", p->i_mtime);
-	PRINT4("i_dtime:       ", p->i_dtime);
-	PRINT2("i_gid:         ", p->i_gid);
-	PRINT2("i_links_count: ", p->i_links_count);
-	PRINT4("i_blocks:      ", p->i_blocks);
-	PRINT4("i_block[0]:    ", p->i_block[0]);
-	PRINT4("i_block[1]:    ", p->i_block[1]);
-	PRINT4("i_block[2]:    ", p->i_block[2]);
-	PRINT4("i_block[3]:    ", p->i_block[3]);
-	PRINT4("i_block[4]:    ", p->i_block[4]);
-	PRINT4("i_block[5]:    ", p->i_block[5]);
-	PRINT4("i_block[6]:    ", p->i_block[6]);
-	PRINT4("i_block[7]:    ", p->i_block[7]);
-	PRINT4("i_block[8]:    ", p->i_block[8]);
-	PRINT4("i_block[9]:    ", p->i_block[9]);
-	PRINT4("i_block[10]:   ", p->i_block[10]);
-	PRINT4("i_block[11]:   ", p->i_block[11]);
-	PRINT4("i_block[12]:   ", p->i_block[12]);
-	PRINT4("i_block[13]:   ", p->i_block[13]);
-	PRINT4("i_block[14]:   ", p->i_block[14]);
-	PRINT4("i_flags:       ", p->i_flags);
-	PRINT4("i_generation:  ", p->i_generation);
-	PRINT4("i_file_acl:    ", p->i_file_acl);
-	PRINT4("i_dir_acl:     ", p->i_dir_acl);
-	PRINT4("i_faddr:       ", p->i_faddr);
+	PRINT2("i_mode:        ", le16_to_cpu(p->i_mode));
+	PRINT2("i_uid:         ", le16_to_cpu(p->i_uid));
+	PRINT4("i_size:        ", le32_to_cpu(p->i_size));
+	PRINT4("i_atime:       ", le32_to_cpu(p->i_atime));
+	PRINT4("i_ctime:       ", le32_to_cpu(p->i_ctime));
+	PRINT4("i_mtime:       ", le32_to_cpu(p->i_mtime));
+	PRINT4("i_dtime:       ", le32_to_cpu(p->i_dtime));
+	PRINT2("i_gid:         ", le16_to_cpu(p->i_gid));
+	PRINT2("i_links_count: ", le16_to_cpu(p->i_links_count));
+	PRINT4("i_blocks:      ", le32_to_cpu(p->i_blocks));
+	PRINT4("i_block[0]:    ", le32_to_cpu(p->i_block[0]));
+	PRINT4("i_block[1]:    ", le32_to_cpu(p->i_block[1]));
+	PRINT4("i_block[2]:    ", le32_to_cpu(p->i_block[2]));
+	PRINT4("i_block[3]:    ", le32_to_cpu(p->i_block[3]));
+	PRINT4("i_block[4]:    ", le32_to_cpu(p->i_block[4]));
+	PRINT4("i_block[5]:    ", le32_to_cpu(p->i_block[5]));
+	PRINT4("i_block[6]:    ", le32_to_cpu(p->i_block[6]));
+	PRINT4("i_block[7]:    ", le32_to_cpu(p->i_block[7]));
+	PRINT4("i_block[8]:    ", le32_to_cpu(p->i_block[8]));
+	PRINT4("i_block[9]:    ", le32_to_cpu(p->i_block[9]));
+	PRINT4("i_block[10]:   ", le32_to_cpu(p->i_block[10]));
+	PRINT4("i_block[11]:   ", le32_to_cpu(p->i_block[11]));
+	PRINT4("i_block[12]:   ", le32_to_cpu(p->i_block[12]));
+	PRINT4("i_block[13]:   ", le32_to_cpu(p->i_block[13]));
+	PRINT4("i_block[14]:   ", le32_to_cpu(p->i_block[14]));
+	PRINT4("i_flags:       ", le32_to_cpu(p->i_flags));
+	PRINT4("i_generation:  ", le32_to_cpu(p->i_generation));
+	PRINT4("i_file_acl:    ", le32_to_cpu(p->i_file_acl));
+	PRINT4("i_dir_acl:     ", le32_to_cpu(p->i_dir_acl));
+	PRINT4("i_faddr:       ", le32_to_cpu(p->i_faddr));
 	PRINT1("l_i_frag:      ", p->osd2.linux2.l_i_frag);
 	PRINT1("l_i_fsize:     ", p->osd2.linux2.l_i_fsize);
-	PRINT2("l_i_uid_high:  ", p->osd2.linux2.l_i_uid_high);
-	PRINT2("l_i_gid_high:  ", p->osd2.linux2.l_i_gid_high);
+	PRINT2("l_i_uid_high:  ", le16_to_cpu(p->osd2.linux2.l_i_uid_high));
+	PRINT2("l_i_gid_high:  ", le16_to_cpu(p->osd2.linux2.l_i_gid_high));
 	printf("\n");
 #endif
 }
@@ -420,16 +429,16 @@ int getInode(t_prog_const* prog_c, t_e2info* fs, t_chk* chk,
 	t_inode_cache		*cache = &fs->inode_cache;
 
 	if (prog_c->liveinfo_fd < 0 &&
-	    (ino >= fs->sb.s_first_ino || ino == EXT2_ROOT_INO))
+	    (ino >= le32_to_cpu(fs->sb.s_first_ino) || ino == EXT2_ROOT_INO))
 		inode_read = TRUE;
 	ino--;
-	gn = ino / fs->sb.s_inodes_per_group;
-	offset = ino % fs->sb.s_inodes_per_group;
+	gn = ino / le32_to_cpu(fs->sb.s_inodes_per_group);
+	offset = ino % le32_to_cpu(fs->sb.s_inodes_per_group);
 
-	sz_inode = fs->sb.s_inode_size;
+	sz_inode = le16_to_cpu(fs->sb.s_inode_size);
 	gd = &fs->gd[gn];
 	ipb = fs->bs / sz_inode;
-	bn = gd->bg_inode_table + offset / ipb;
+	bn = le32_to_cpu(gd->bg_inode_table) + offset / ipb;
 
 	if (inode_read) {
 		unsigned int	cnt, hit = 0;
@@ -445,7 +454,7 @@ int getInode(t_prog_const* prog_c, t_e2info* fs, t_chk* chk,
 #endif
 		if (!hit) {
 			pos = (off_t)bn * fs->bs + (offset % ipb) * sz_inode;
-			cnt = fs->sb.s_inodes_per_group - offset;
+			cnt = le32_to_cpu(fs->sb.s_inodes_per_group) - offset;
 			if (cnt > INODE_CACHE_MAX)
 				cnt = INODE_CACHE_MAX;
 			rc = uSeekRead(prog_c->fd, pos, cache->p,
@@ -466,7 +475,7 @@ int getInode(t_prog_const* prog_c, t_e2info* fs, t_chk* chk,
 
 		i_info->ino = ino;
 		i_info->fblocks = 0;
-		i_info->bn_min = fs->sb.s_blocks_count;
+		i_info->bn_min = le32_to_cpu(fs->sb.s_blocks_count);
 		i_info->bn_max = 0;
 		i_info->sfrags = 0;
 		i_info->frags = 0;
@@ -524,7 +533,7 @@ int chkBlockNum(t_prog_const* prog_c, t_e2info* fs, t_chk* chk,
 
 int chkXattrBlockNum(t_prog_const* prog_c, t_e2info* fs, t_chk* chk,
 		     t_inode_info* i_info) {
-	unsigned int	bn = i_info->inode.i_file_acl;
+	unsigned int	bn = le32_to_cpu(i_info->inode.i_file_acl);
 	t_bchk		*bchk = &chk->blocks[bn];
 
 	if (bchk->in_proc && bchk->type != BT_XATTR) {
@@ -582,7 +591,6 @@ int chkBlock(t_prog_const* prog_c, t_e2info* fs, t_chk* chk,
 	unsigned int*			p;
 	struct ext2_dir_entry_2*	de;
 
-
 	if (found) {
 		rc = chkBlockNum(prog_c, fs, chk, i_info, bn);
 		if (rc == ERROR)
@@ -598,7 +606,9 @@ int chkBlock(t_prog_const* prog_c, t_e2info* fs, t_chk* chk,
 		max = fs->bs / sizeof(unsigned int); /*? __u32 ?*/
 		ind_nest--;
 		for (i = 0; i < max; i++) {
+/*			bn = le32_to_cpu(*p++); */
 			bn = *p++;
+			
 			if (bn) {
 				rc = chkBlock(prog_c, fs, chk, i_info,
 					      bn, found, ind_nest);
@@ -606,7 +616,7 @@ int chkBlock(t_prog_const* prog_c, t_e2info* fs, t_chk* chk,
 					return ERROR;
 			}
 		}
-	} else if (S_ISDIR(i_info->inode.i_mode)) {
+	} else if (S_ISDIR(le16_to_cpu(i_info->inode.i_mode))) {
 		if (found && prog_c->not_follow)
 			return NEXT;
 		if (READ_BLOCK(prog_c, fs, bn, buf) < 0)
@@ -622,9 +632,9 @@ int chkBlock(t_prog_const* prog_c, t_e2info* fs, t_chk* chk,
 			chk->max_depth = depth;
 
 		rc = NEXT;
-		for (i = 0; i < fs->bs; i += de->rec_len) {
+		for (i = 0; i < fs->bs; i += le16_to_cpu(de->rec_len)) {
 			de = (struct ext2_dir_entry_2*)(buf + i);
-			if (!de->rec_len)
+			if (!le16_to_cpu(de->rec_len))
 				break;
 			strncpy(chk->name[depth], de->name, de->name_len);
 			chk->name[depth][de->name_len] = '\0';
@@ -637,7 +647,7 @@ int chkBlock(t_prog_const* prog_c, t_e2info* fs, t_chk* chk,
 				       found, &new_found))
 				continue;
 
-			rc = chkInode(prog_c, fs, chk, de->inode, new_found);
+			rc = chkInode(prog_c, fs, chk, le32_to_cpu(de->inode), new_found);
 			if (!found || rc == NOT_FOUND || rc == ERROR)
 				break;
 		}
@@ -649,7 +659,7 @@ int chkBlock(t_prog_const* prog_c, t_e2info* fs, t_chk* chk,
 int isSymLinkWithNoBlocks(t_e2info* fs, struct ext2_inode* inode) {
 	unsigned int		i_blocks = inode->i_blocks;
 
-	if (inode->i_file_acl)
+	if (le32_to_cpu(inode->i_file_acl))
 		/* only for now. i_file_acl use only one block. */
 		i_blocks -= fs->bs / 512;
 	return (i_blocks == 0);
@@ -662,7 +672,7 @@ int chkIBlocksUnuseDevdrv(t_prog_const* prog_c, t_e2info* fs, t_chk* chk,
 	unsigned char		found_at_child;
 
 	inode = &i_info->inode;
-	i_info->is_dir = S_ISDIR(inode->i_mode);
+	i_info->is_dir = S_ISDIR(le16_to_cpu(inode->i_mode));
 	if (isSymLinkWithNoBlocks(fs, inode)) {
 		rc = FOUND;
 		goto EXIT;
@@ -671,28 +681,28 @@ int chkIBlocksUnuseDevdrv(t_prog_const* prog_c, t_e2info* fs, t_chk* chk,
 	found_at_child = found ? FOUND : NOT_FOUND;
 
 	for (i = 0; i < EXT2_NDIR_BLOCKS; i++)
-		if ((bn = inode->i_block[i])) {
+		if ((bn = le32_to_cpu(inode->i_block[i]))) {
 			rc = chkBlock(prog_c, fs, chk, i_info, bn, found, 0);
 			if (rc == FOUND)
 				found_at_child = FOUND;
 			else if (rc != NEXT)
 				goto EXIT;
 		}
-	if ((bn = inode->i_block[EXT2_IND_BLOCK])) {
+	if ((bn = le32_to_cpu(inode->i_block[EXT2_IND_BLOCK]))) {
 		rc = chkBlock(prog_c, fs, chk, i_info, bn, found, 1);
 		if (rc == FOUND)
 			found_at_child = FOUND;
 		else if (rc != NEXT)
 			goto EXIT;
 	}
-	if ((bn = inode->i_block[EXT2_DIND_BLOCK])) {
+	if ((bn = le32_to_cpu(inode->i_block[EXT2_DIND_BLOCK]))) {
 		rc = chkBlock(prog_c, fs, chk, i_info, bn, found, 2);
 		if (rc == FOUND)
 			found_at_child = FOUND;
 		else if (rc != NEXT)
 			goto EXIT;
 	}
-	if ((bn = inode->i_block[EXT2_TIND_BLOCK])) {
+	if ((bn = le32_to_cpu(inode->i_block[EXT2_TIND_BLOCK]))) {
 		rc = chkBlock(prog_c, fs, chk, i_info, bn, found, 3);
 		if (rc == FOUND)
 			found_at_child = FOUND;
@@ -758,7 +768,8 @@ int chkFileBlocks(t_prog_const* prog_c, t_e2info* fs, t_chk* chk,
 		for (fbinfo = (struct fblocks_info*)p;
 		     (char*)fbinfo < p + len; fbinfo++) {
 			if (fbinfo == (struct fblocks_info*)p) {
-				i_info->inode.i_file_acl = fbinfo->bn;
+/*				i_info->inode.i_file_acl = cpu_to_le32(fbinfo->bn); */
+				i_info->inode.i_file_acl = fbinfo->bn; 
 				continue;
 			}
 			bn = fbinfo->bn;
@@ -863,7 +874,7 @@ int chkInode(t_prog_const* prog_c, t_e2info* fs, t_chk* chk, unsigned int ino,
 #endif
 	i_info.inode.i_file_acl = 0;
 
-	if (ino == 0 || ino > fs->sb.s_inodes_count)
+	if (ino == 0 || ino > le32_to_cpu(fs->sb.s_inodes_count))
 		/* already deleted file */
 		return FOUND;
 
@@ -871,12 +882,11 @@ int chkInode(t_prog_const* prog_c, t_e2info* fs, t_chk* chk, unsigned int ino,
 	if (chk->inodes[ino] == USED_INODE && ino != EXT2_ROOT_INO)
 		/* hard link */
 		return FOUND;
-
 	rc = getInode(prog_c, fs, chk, ino, &i_info, found);
 	if (rc == ERROR)
 		goto EXIT;
 
-	if (ino < fs->sb.s_first_ino && ino != EXT2_ROOT_INO) {
+	if (ino < le32_to_cpu(fs->sb.s_first_ino) && ino != EXT2_ROOT_INO) {
 		rc = FOUND;
 		goto EXIT;
 	}
@@ -885,6 +895,7 @@ int chkInode(t_prog_const* prog_c, t_e2info* fs, t_chk* chk, unsigned int ino,
 	else
 		rc = chkIBlocksUseDevdrv(prog_c, fs, chk, &i_info, found);
 
+
 EXIT:
 #ifdef DEBUG
 	printf("<<< %u, %u\n", found, rc);
@@ -900,9 +911,8 @@ EXIT:
 			return rc;
 	}
 	i_info.xattrs = 0;
-	if (i_info.inode.i_file_acl && ino >= fs->sb.s_first_ino)
+	if (le32_to_cpu(i_info.inode.i_file_acl) && ino >= le32_to_cpu(fs->sb.s_first_ino))
 		chkXattrBlockNum(prog_c, fs, chk, &i_info);
-
 	if (rc == ERROR) {
 		/* error on the mounted partition */
 		i_info.fblocks = 0;
@@ -919,7 +929,6 @@ EXIT:
 		chk->sfrags += i_info.sfrags;
 		chk->frags += i_info.frags;
 	}
-
 	/* update chk->blocks */
 	is_fragment = (i_info.sfrags || i_info.frags);
 	if (is_fragment)
@@ -1017,7 +1026,7 @@ int chkAllInodes(t_prog_const* prog_c, t_e2info* fs, t_chk* chk) {
 		}
 	}
 	if (prog_c->liveinfo_fd < 0) {
-		fs->inode_cache.p = uCalloc(fs->sb.s_inode_size,
+		fs->inode_cache.p = uCalloc(le16_to_cpu(fs->sb.s_inode_size),
 					    INODE_CACHE_MAX);
 		if (fs->inode_cache.p == NULL)
 			return ERROR;
@@ -1035,7 +1044,7 @@ int chkAllInodes(t_prog_const* prog_c, t_e2info* fs, t_chk* chk) {
 	}
 
 	if (prog_c->mode == CHK_PARTITION)
-		for (i = 1; i < fs->sb.s_first_ino; i++) {
+		for (i = 1; i < le32_to_cpu(fs->sb.s_first_ino); i++) {
 			if (i == EXT2_ROOT_INO)
 				continue;
 			rc = chkInode(prog_c, fs, chk, i, found);
@@ -1050,7 +1059,7 @@ int chkAllInodes(t_prog_const* prog_c, t_e2info* fs, t_chk* chk) {
 
 	if (prog_c->mode == CHK_PARTITION) {
 		chk->bn_min = 0;
-		chk->bn_max = fs->sb.s_blocks_count - 1;
+		chk->bn_max = le32_to_cpu(fs->sb.s_blocks_count) - 1;
 	}
 
 	if (prog_c->verbose_level >= 1)
@@ -1070,19 +1079,19 @@ int chkBlockBitmap(t_prog_const* prog_c, t_e2info* fs, t_chk* chk) {
 	int			err = FALSE;
 
 	sb = &fs->sb;
-	bpg = sb->s_blocks_per_group;
-	bn = sb->s_first_data_block;
+	bpg = le32_to_cpu(sb->s_blocks_per_group);
+	bn = le32_to_cpu(sb->s_first_data_block);
 	bchk = &chk->blocks[bn];
 	for (gn = 0; gn < fs->gc; gn++) {
 		gd = &fs->gd[gn];
 
 		/* read block bitmap */
-		rc = READ_BLOCK(prog_c, fs, gd->bg_block_bitmap, buf);
+		rc = READ_BLOCK(prog_c, fs, le32_to_cpu(gd->bg_block_bitmap), buf);
 		if (rc < 0)
 			return ERROR;
 
 		/* block compare check */
-		for (i = 0; i < bpg && bn < sb->s_blocks_count; i++) {
+		for (i = 0; i < bpg && bn < le32_to_cpu(sb->s_blocks_count); i++) {
 			if (i % 8 == 0)
 				mask = 1;
 			else
@@ -1122,19 +1131,19 @@ int chkInodeBitmap(t_prog_const* prog_c, t_e2info* fs, t_chk* chk) {
 	int			err = FALSE;
 
 	sb = &fs->sb;
-	ipg = sb->s_inodes_per_group;
+	ipg = le32_to_cpu(sb->s_inodes_per_group);
 	ichk = chk->inodes;
 	ino = 1;
 	for (gn = 0; gn < fs->gc; gn++) {
 		gd = &fs->gd[gn];
 
 		/* read inode bitmap */
-		rc = READ_BLOCK(prog_c, fs, gd->bg_inode_bitmap, buf);
+		rc = READ_BLOCK(prog_c, fs, le32_to_cpu(gd->bg_inode_bitmap), buf);
 		if (rc < 0)
 			return ERROR;
 
 		/* inode compare check */
-		for (i = 0; i < ipg && ino <= sb->s_inodes_count; i++) {
+		for (i = 0; i < ipg && ino <= le32_to_cpu(sb->s_inodes_count); i++) {
 			if (i % 8 == 0)
 				mask = 1;
 			else
@@ -1356,7 +1365,7 @@ int outputResult(t_prog_const* prog_c, t_e2info* fs, t_chk* chk) {
 
 	printf("------ cdavl result start ------\n");
 	printf("fstype\t%s\n",
-	       (fs->sb.s_feature_compat & EXT3_FEATURE_COMPAT_HAS_JOURNAL) ?
+	       (le32_to_cpu(fs->sb.s_feature_compat) & EXT3_FEATURE_COMPAT_HAS_JOURNAL) ?
 	       "ext3" : "ext2");
 	printf("mount\t%s", IS_MOUNT(fs) ? "mount" : "unmount");
 	if (prog_c->liveinfo_fd >= 0)
@@ -1553,10 +1562,10 @@ int main(int argc, char* argv[]) {
 	if (rc == ERROR)
 		goto ERR_EXIT;
 
-	fs.bs = EXT2_MIN_BLOCK_SIZE << fs.sb.s_log_block_size;
-	fs.gc = (fs.sb.s_blocks_count - fs.sb.s_first_data_block +
-		   fs.sb.s_blocks_per_group - 1) / fs.sb.s_blocks_per_group;
-	fs.sysb_info = uCalloc(sizeof(t_sys_blk_info), fs.gc * 2);
+	fs.bs = EXT2_MIN_BLOCK_SIZE << le32_to_cpu(fs.sb.s_log_block_size);
+	fs.gc = (le32_to_cpu(fs.sb.s_blocks_count) - le32_to_cpu(fs.sb.s_first_data_block) +
+		   le32_to_cpu(fs.sb.s_blocks_per_group) - 1) / le32_to_cpu(fs.sb.s_blocks_per_group);
+	fs.sysb_info = uCalloc(sizeof(t_sys_blk_info), fs.gc * 2); 
 					/* '2' is for sparse super */
 	if (fs.sysb_info == NULL)
 		goto ERR_EXIT;
@@ -1571,18 +1580,18 @@ int main(int argc, char* argv[]) {
 		goto ERR_EXIT;
 
 	/* initialize chk variable */
-	chk.blocks = uMalloc(sizeof(t_bchk) * fs.sb.s_blocks_count);
+	chk.blocks = uMalloc(sizeof(t_bchk) * le32_to_cpu(fs.sb.s_blocks_count));
 	if (chk.blocks == NULL)
 		goto ERR_EXIT;
-	memset(chk.blocks, BT_UNUSED_BLOCK, fs.sb.s_blocks_count);
-	chk.inodes = uMalloc(sizeof(unsigned char) * fs.sb.s_inodes_count);
+	memset(chk.blocks, BT_UNUSED_BLOCK, le32_to_cpu(fs.sb.s_blocks_count));
+	chk.inodes = uMalloc(sizeof(unsigned char) * le32_to_cpu(fs.sb.s_inodes_count));
 	if (chk.inodes == NULL)
 		goto ERR_EXIT;
-	memset(chk.inodes, UNUSED_INODE, fs.sb.s_inodes_count);
+	memset(chk.inodes, UNUSED_INODE, le32_to_cpu(fs.sb.s_inodes_count));
 	chk.p_fpath = prog_c.fpath;
 	chk.depth = 0;
 	chk.path_found = FALSE;
-	chk.bn_min = fs.sb.s_blocks_count;
+	chk.bn_min = le32_to_cpu(fs.sb.s_blocks_count);
 	chk.bn_max = 0;
 
 	/* check system block */
