@@ -5,7 +5,7 @@
 
 /*****************************************************************************/
 /*  liveinfo.c - gathering live information (ext2/3 common function)         */
-/*  Copyright: Copyright (c) Hitachi, Ltd. 2004-2007                         */
+/*  Copyright: Copyright (c) Hitachi, Ltd. 2004-2008                         */
 /*             Authors: Yumiko Sugita (yumiko.sugita.yf@hitachi.com),        */
 /*                      Satoshi Fujiwara (sa-fuji@sdl.hitachi.co.jp)         */
 /*                                                                           */
@@ -36,6 +36,9 @@
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
 #include <linux/buffer_head.h>
 #endif
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 26)
+#include <linux/fdtable.h>
+#endif
 
 
 MODULE_AUTHOR("Satoshi Fujiwara");
@@ -227,11 +230,7 @@ int liveinfo_init(void) {
 }
 
 void liveinfo_exit(void) {
-	int	rc;
-
-	rc = unregister_chrdev(DEVICE_MAJOR, DEVICE_NAME);
-	if (rc < 0)
-		printk("DAVL_LIVEINFO: unregister_chrdev failed.(%d)\n", rc);
+	unregister_chrdev(DEVICE_MAJOR, DEVICE_NAME);
 }
 
 module_init(liveinfo_init);
