@@ -5,8 +5,8 @@
 
 /*****************************************************************************/
 /*  gdavl.c - GUI disk allocation viewer                                     */
-/*  Copyright: Copyright (c) Hitachi, Ltd. 2004-2005                         */
-/*             Authors: Yumiko Sugita (sugita@sdl.hitachi.co.jp),            */
+/*  Copyright: Copyright (c) Hitachi, Ltd. 2004-2007                         */
+/*             Authors: Yumiko Sugita (yumiko.sugita.yf@hitachi.com),        */
 /*                      Satoshi Fujiwara (sa-fuji@sdl.hitachi.co.jp)         */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
@@ -1756,7 +1756,7 @@ guint on_input(GIOChannel *c, GIOCondition condition, gpointer tmp,
 	}
 	*/
 
-	rc = g_io_channel_read(c, buf, max, &cnt);
+	rc = g_io_channel_read(c, buf, max, (unsigned int*)&cnt);
 	if (rc == G_IO_ERROR_AGAIN)
 		return TRUE;
 	if (rc != G_IO_ERROR_NONE || cnt == 0) {
